<?php

/**
 *
 * liveSite - Enterprise Website Platform
 *
 * @author      Camelback Web Architects
 * @link        https://livesite.com
 * @copyright   2001-2017 Camelback Consulting, Inc.
 * @license     https://opensource.org/licenses/mit-license.html MIT License
 *
 */

include('init.php');
$user = validate_user();
validate_ecommerce_access($user);

$liveform = new liveform('add_key_code');

if (!$_POST) {

    // If the form has not been submitted yet, then pre-populate fields with data.
    if (!$liveform->field_in_session('name')) {
        $liveform->assign_field_value('enabled', 1);
    }

    $screen = 'create';

    echo output_header();

    require('templates/edit_key_code.php');

    echo output_footer();

    $liveform->remove_form();

} else {

    validate_token_field();

    $liveform->add_fields_to_session();
    
    $liveform->validate_required_field('code', 'Key Code is required.');
    $liveform->validate_required_field('offer_code', 'Offer Code is required.');

    // If there is not already an error for the code field,
    // and that code is already in use, then output error.
    if (
        (!$liveform->check_field_error('code'))
        && (db_value("SELECT COUNT(*) FROM key_codes WHERE code = '" . e($liveform->get_field_value('code')) . "'") != 0)
    ) {
        $liveform->mark_error('code', 'Sorry, the key code that you entered is already in use, so please enter a different key code.');
    }

    // If there is not already an error for the offer code field,
    // and an offer cannot be found for that code, then output error.
    if (
        (!$liveform->check_field_error('offer_code'))
        && (db_value("SELECT COUNT(*) FROM offers WHERE code = '" . e($liveform->get_field_value('offer_code')) . "'") == 0)
    ) {
        $liveform->mark_error('offer_code', 'Sorry, we cannot find an offer for the offer code that you entered.');
    }

    $expiration_date = $liveform->get_field_value('expiration_date');

    // If an expiration date was entered and it is not valid, then add error.
    if (($expiration_date != '') && (validate_date($expiration_date) == false)) {
        $liveform->mark_error('expiration_date', 'Please enter a valid expiration date.');
    }

    if ($liveform->check_form_errors()) {
        go($_SERVER['PHP_SELF']);
    }

    db(
        "INSERT INTO key_codes (
            code,
            offer_code,
            enabled,
            expiration_date,
            single_use,
            user,
            timestamp)
        VALUES (
            '" . e($liveform->get_field_value('code')) . "',
            '" . e($liveform->get_field_value('offer_code')) . "',
            '" . e($liveform->get_field_value('enabled')) . "',
            '" . e(prepare_form_data_for_input($expiration_date, 'date')) . "',
            '" . e($liveform->get_field_value('single_use')) . "',
            '" . USER_ID . "',
            UNIX_TIMESTAMP())");

    log_activity('key code (' . $liveform->get_field_value('code') . ') was created', $_SESSION['sessionusername']);

    $liveform_view_key_codes = new liveform('view_key_codes');
    $liveform_view_key_codes->add_notice('The key code has been created.');

    $liveform->remove_form();

    go(PATH . SOFTWARE_DIRECTORY . '/view_key_codes.php');

}

?>