<?php

/**
 *
 * liveSite - Enterprise Website Platform
 *
 * @author      Camelback Web Architects
 * @link        https://livesite.com
 * @copyright   2001-2017 Camelback Consulting, Inc.
 * @license     https://opensource.org/licenses/mit-license.html MIT License
 *
 */

include('init.php');

$form = new liveform('change_random_password');

// if the user is not logged in, then output error
if (validate_login($_SESSION['sessionusername'], $_SESSION['sessionpassword']) == false) {
    log_activity('access denied for username (' . $_SESSION['sessionusername'] . ')', 'unknown');
    output_error('Invalid login. <a href="javascript:history.go(-1)">Go back</a>.');
}

validate_token_field();

$form->add_fields_to_session();

// validate required fields
$form->validate_required_field('new_password', 'New Password is required.');
$form->validate_required_field('new_password_verify', 'Please type new password again.');

// if there is not already an error for the new password fields, check to see if new password and new password verify do not match
if (($form->check_field_error('new_password') == false) && ($form->check_field_error('new_password_verify') == false)) {
    if ($form->get_field_value('new_password') != $form->get_field_value('new_password_verify')) {
        $form->mark_error('new_password', 'The two new passwords you entered did not match.');
        $form->mark_error('new_password_verify');
        $form->assign_field_value('new_password', '');
        $form->assign_field_value('new_password_verify', '');
    }
}

// if there is not already an error for the new password field,
// and there is not already an error for the new password verify field,
// and strong password is enabled,
// and the new password is not strong,
// then mark error for new password fields
if (
    ($form->check_field_error('new_password') == false)
    && ($form->check_field_error('new_password_verify') == false)
    && (STRONG_PASSWORD == true)
    && (validate_password_strength($form->get_field_value('new_password')) == false)
) {
    $form->mark_error('new_password', 'The new password you entered does not meet the requirements. Please enter a different new password.');
    $form->mark_error('new_password_verify');
    $form->assign_field_value('new_password', '');
    $form->assign_field_value('new_password_verify', '');
}

// Check to see if the new password is in the password hint, and mark an error if so.
if (($form->get_field_value('password_hint') != '') && ($form->get_field_value('new_password') != '')) {
    if (
        ($form->get_field_value('password_hint') == $form->get_field_value('new_password'))
         || (mb_strpos(mb_strtolower($form->get_field_value('password_hint')), mb_strtolower($form->get_field_value('new_password'))) !== false) 
       ) {
            $form->mark_error('password_hint', 'Your password hint cannot contain your password.');
            $form->assign_field_value('password_hint', '');
    }
}

// if an error exists, then send user back to previous screen
if ($form->check_form_errors()) {
    go(get_page_type_url('change random password'));
}

// update password for user
$query = 
    "UPDATE user 
    SET 
        user_password = '" . escape(md5($form->get_field_value('new_password'))) . "',
        user_random_password = '0',
        user_password_hint = '" . escape($form->get_field_value('password_hint')) . "'
    WHERE 
        (user_username = '" . escape($_SESSION['sessionusername']) . "')
        AND (user_password = '" . escape($_SESSION['sessionpassword']) . "')";
$result = mysqli_query(db::$con, $query) or output_error('Query failed');

// assign new password to session password
$_SESSION['sessionpassword'] = md5($form->get_field_value('new_password'));
log_activity("user changed random password", $_SESSION['sessionusername']);

// If remember me is on and the user has chosen to be remembered,
// and the user is not logged in as a different user,
// then update cookie with new login information.
if (
    (REMEMBER_ME == true)
    && (isset($_COOKIE['software']['username']) == true)
    && ($_SESSION['software']['logged_in_as_different_user'] == false)
) {
    $secure = false;

    // If secure mode is enabled, then prepare secure cookie values.
    if (URL_SCHEME == 'https://') {
        $secure = true;
    }

    // If PHP version is greater than or equal to 5.2.0 then add cookies
    // for login info so that user will be logged in automatically and also
    // use httponly cookie, in order to prevent hacking methods.  PHP before 5.2.0
    // does not support setting httponly cookies.
    if (version_compare(PHP_VERSION, '5.2.0', '>=') == TRUE) {
        setcookie('software[password]', md5($form->get_field_value('new_password')), time() + 315360000, '/', '', $secure, true);

    // Otherwise store login info in cookies without httponly cookie.
    } else {
        setcookie('software[password]', md5($form->get_field_value('new_password')), time() + 315360000, '/', '', $secure);
    }
}

// get user information
$user = validate_user();

// get home page name
$home_page_name = get_page_name($user['home']);

$send_to = $form->get('send_to');

$form->remove();

$form->set('screen', 'confirm');

// if there is a send to, then forward user to send to
if ($send_to != '') {
    $continue_url = $send_to;
    
// else if user has a home page, then forward user to that page
} elseif ($home_page_name != '') {
    $continue_url = PATH . encode_url_path($home_page_name);

// else if user's role is administrator, designer, or manager
// or user has edit rights
// or user has access to control panel
// then forward user to control panel welcome screen
} elseif (
    ($user['role'] < 3)
    || (no_acl_check($user['id']) == true)
    || ($user['manage_calendars'] == true)
    || ($user['manage_forms'] == true)
    || ($user['manage_visitors'] == true)
    || ($user['manage_contacts'] == true)
    || ($user['manage_emails'] == true)
    || ($user['manage_ecommerce'] == true)
    || $user['manage_ecommerce_reports']
    || (count(get_items_user_can_edit('ad_regions', $user['id'])) > 0)
) {
    $continue_url = PATH . SOFTWARE_DIRECTORY . '/welcome.php';
    
// else we do not know where to send the user, so send user to home page.
} else {
    $continue_url = PATH;
}

$form->add_notice('Your password has been changed. <a href="' . h(escape_url($continue_url)) . '">Continue</a>');

go(get_page_type_url('change random password'));